using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using BEPUphysics;
using BEPUphysics.Entities;
using BEPUphysics.DataStructures;
using BEPUphysicsDrawer.Models;

namespace Bowling
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class BowlingGame : Microsoft.Xna.Framework.Game
    {
        SpriteBatch spriteBatch;

        public Camera Camera;

        public Space physicsSpace;

        ModelDrawer modelDrawer;

        Model bowlModel;
        Model floorModel;
        Model ballModel;

        Cylinder bowl;
        Box floor;
        Sphere ball;

        public BowlingGame()
        {
            Configuration.Graphics = new GraphicsDeviceManager(this);
            Configuration.Graphics.PreferMultiSampling = true;
            Configuration.Graphics.ApplyChanges();

            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // Initialize camera
            Camera = new Camera();

            // Initialize physics
            physicsSpace = new Space();

            physicsSpace.simulationSettings.motionUpdate.gravity = new Vector3(0, -9.81f, 0);

            // Initialize entity types, position and sizes
            bowl = new Cylinder(new Vector3(0.0f, 5.0f, -70.0f), 50.0f, 15.0f, 109.0f);
            floor = new Box(Vector3.Zero, 100.0f, 1.0f, 40.0f);
            ball = new Sphere(new Vector3(0.0f, 2.0f, 100.0f), 5.0f, 15.0f);

            ball.linearVelocity = -Vector3.UnitZ * 50.0f;
            
            physicsSpace.add(bowl);
            physicsSpace.add(floor);
            physicsSpace.add(ball);

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            modelDrawer = new BruteModelDrawer(this);
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // Load models
            floorModel = Content.Load<Model>("floor");
            bowlModel = Content.Load<Model>("bowl");
            ballModel = Content.Load<Model>("tierra");
            
            Matrix scalingBowl = Matrix.CreateScale(1, 1, 1);
            Matrix scalingBall = Matrix.CreateScale(1.0f);

            EntityModel bowlEM = new EntityModel(bowl, bowlModel, scalingBowl, this);
            EntityModel ballEM = new EntityModel(ball, ballModel, scalingBall, this);
           
            //TODO: Uncomment to see the previous models
            //Components.Add(bowlEM);
            //Components.Add(ballEM);

            //Create a physical environment from a triangle mesh.
            //First, collect the the mesh data from the model using a helper function.
            //This special kind of vertex inherits from the TriangleMeshVertex and optionally includes
            //friction/bounciness data.
            //The StaticTriangleGroup requires that this special vertex type is used in lieu of a normal TriangleMeshVertex array.
            StaticTriangleGroup.StaticTriangleGroupVertex[] vertices;
            int[] indices;
            StaticTriangleGroup.getVerticesAndIndicesFromModel(floorModel, out vertices, out indices);

            //Give the mesh information to a new TriangleMesh.  
            //TriangleMeshes are internally accelerated structures that 
            //help handle raycasts and various other queries on mesh data.
            TriangleMesh triangleMesh = new TriangleMesh(vertices, indices);
            //Create the StaticTriangleGroup based on the triangle mesh.
            StaticTriangleGroup triangleGroup = new StaticTriangleGroup(triangleMesh);
            //Scoot the mesh down so its below the previously created kinematic box.
            triangleGroup.worldMatrix = Matrix.CreateTranslation(new Vector3(0, 0, 0));
            modelDrawer.add(triangleGroup.triangleMesh);
            //Add it to the space!
            physicsSpace.add(triangleGroup);
            //Make it visible too.
            //TODO: Uncomment to see the previous model
            //Components.Add(new StaticModel(floorModel, triangleGroup.worldMatrix, this));

            foreach (Entity e in physicsSpace.entities)
            {
                if ((string)e.tag != "noDisplayObject")
                {
                    modelDrawer.add(e);
                }
                else//Remove the now unnecessary tag.
                    e.tag = null;
            }
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            Content.Unload();
            GC.Collect();
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            physicsSpace.update(gameTime);
            Camera.UpDateCam();

            modelDrawer.update();

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            modelDrawer.draw(Camera.View, Camera.Projection);
            // TODO: Add your drawing code here

            base.Draw(gameTime);
        }
    }
}
